using CanonSharp.Combinator.Abstractions;

namespace CanonSharp.Combinator.Parsers.Bases;

/// <summary>
/// 选择解析器
/// 如果第一个不成功则调用第二个
/// </summary>
/// <param name="first">第一个解析器</param>
/// <param name="second">第二个解析器</param>
/// <typeparam name="TToken">输入流类型</typeparam>
/// <typeparam name="T">解析器结果类型</typeparam>
internal sealed class AlternativeParser<TToken, T>(Parser<TToken, T> first, Parser<TToken, T> second)
    : Parser<TToken, T>
{
    internal override ParseResult<TToken, TResult> Run<TState, TResult>(TState state,
        Func<ParseResult<TToken, T>, ParseResult<TToken, TResult>> continuation)
    {
        return first.Run(state, result => result.CaseOf(continuation, _ => second.Run(state, continuation)));
    }
}
