using CanonSharp.Combinator.Abstractions;

namespace CanonSharp.Combinator.Parsers.Bases;

/// <summary>
/// 下一步解析器
/// </summary>
/// <param name="parser">上游解析器</param>
/// <param name="successfulParser">成功情况下的解析器函数</param>
/// <param name="failedParser">失败情况下的解析器函数</param>
/// <typeparam name="TToken">输入流类型</typeparam>
/// <typeparam name="TIntermediate">上游解析器结果类型</typeparam>
/// <typeparam name="T">最终解析结果类型</typeparam>
internal sealed class NextParser<TToken, TIntermediate, T>(
    Parser<TToken, TIntermediate> parser,
    Func<TIntermediate, Parser<TToken, T>> successfulParser,
    Func<FailedResult<TToken, TIntermediate>, Parser<TToken, T>> failedParser) : Parser<TToken, T>
{
    internal override ParseResult<TToken, TResult> Run<TState, TResult>(TState state,
        Func<ParseResult<TToken, T>, ParseResult<TToken, TResult>> continuation)
    {
        return parser.Run(state, result => result.CaseOf(
            successfulResult => successfulResult.Next(successfulParser, continuation),
            failedResult => failedParser(failedResult).Run(state, continuation)));
    }
}
