using CanonSharp.Combinator.Abstractions;

namespace CanonSharp.Combinator.Results;

/// <summary>
/// 实际实现的解析成功结果
/// </summary>
/// <param name="result">解析结果</param>
/// <param name="state">解析成功之后的下一个输入流状态</param>
/// <typeparam name="TToken">输入流类型</typeparam>
/// <typeparam name="TState">输入流的状态类型</typeparam>
/// <typeparam name="T">解析结果的类型</typeparam>
internal sealed class InternalSuccessfulResult<TToken, TState, T>(T result, TState state)
    : SuccessfulResult<TToken, T>(result)
    where TState : IReadState<TToken, TState>
{
    protected override ParseResult<TToken, TResult> RunNext<TNext, TResult>(Parser<TToken, TNext> parser,
        Func<ParseResult<TToken, TNext>, ParseResult<TToken, TResult>> continuation)
        => parser.Run(state, continuation);

    public override ParseResult<TToken, TResult> Map<TResult>(Func<T, TResult> map)
        => new InternalSuccessfulResult<TToken, TState, TResult>(map(Value), state);
}
